import { Data } from './models';
/**
 * Gets the playback state
 *
 * @public
 * @param data - The SpriteSpin instance data
 */
export declare function getPlaybackState(data: Data): PlaybackState;
/**
 * The playback state
 *
 * @public
 */
export interface PlaybackState {
    frameTime: number;
    lastFrame: number;
    lastLane: number;
    handler: number;
}
/**
 * Updates the frame or lane number of the SpriteSpin data.
 *
 * @public
 * @param data - The SpriteSpin instance data
 * @param frame - The frame number to set
 * @param lane - The lane number to set
 */
export declare function updateFrame(data: Data, frame?: number, lane?: number): void;
/**
 * Stops the running animation.
 *
 * @public
 * @param data - The SpriteSpin instance data
 */
export declare function stopAnimation(data: Data): void;
/**
 * Starts animation playback if needed.
 *
 * @remarks
 * Starts animation playback if `animate` property is `true` and the animation is not yet running.
 *
 * @public
 * @param data - The SpriteSpin instance data
 */
export declare function applyAnimation(data: Data): void;
/**
 * Starts the animation playback
 *
 * @remarks
 * Starts the animation playback and also sets the `animate` property to `true`
 *
 * @public
 * @param data - The SpriteSpin instance data
 */
export declare function startAnimation(data: Data): void;
